@@ -38,7 +38,6 @@
 ****************************************************************************/
 
 #include "qv4qmlcontext_p.h"
-#include <private/qv8engine_p.h>
 
 #include <private/qqmlengine_p.h>
 #include <private/qqmlcontext_p.h>
@@ -232,28 +231,32 @@ ReturnedValue QQmlContextWrapper::getPropertyAndBase(const QQmlContextWrapper *r
             } else if (r.type.isValid()) {
                 if (lookup) {
                     if (r.type.isSingleton()) {
-                        QQmlEngine *e = v4->qmlEngine();
-                        QQmlType::SingletonInstanceInfo *siinfo = r.type.singletonInstanceInfo();
-                        siinfo->init(e);
-                        if (siinfo->qobjectApi(e)) {
+                        QQmlEnginePrivate *e = QQmlEnginePrivate::get(v4->qmlEngine());
+                        if (r.type.isQObjectSingleton() || r.type.isCompositeSingleton()) {
+                            e->singletonInstance<QObject*>(r.type);
                             lookup->qmlContextSingletonLookup.singleton =
                                     static_cast<Heap::Object*>(
                                         Value::fromReturnedValue(
                                             QQmlTypeWrapper::create(v4, nullptr, r.type)
                                         ).heapObject());
                         } else {
-                            QV4::ScopedObject o(scope, QJSValuePrivate::convertedToValue(v4, siinfo->scriptApi(e)));
+                            QJSValue singleton = e->singletonInstance<QJSValue>(r.type);
+                            QV4::ScopedObject o(scope, QJSValuePrivate::convertedToValue(v4, singleton));
                             lookup->qmlContextSingletonLookup.singleton = o->d();
                         }
                         lookup->qmlContextPropertyGetter = QQmlContextWrapper::lookupSingleton;
                         return lookup->qmlContextPropertyGetter(lookup, v4, base);
                     }
                 }
-                return QQmlTypeWrapper::create(v4, scopeObject, r.type);
+                result = QQmlTypeWrapper::create(v4, scopeObject, r.type);
             } else if (r.importNamespace) {
-                return QQmlTypeWrapper::create(v4, scopeObject, context->imports, r.importNamespace);
+                result = QQmlTypeWrapper::create(v4, scopeObject, context->imports, r.importNamespace);
             }
-            Q_ASSERT(!"Unreachable");
+            if (lookup) {
+                lookup->qmlTypeLookup.qmlTypeWrapper = static_cast<Heap::Object*>(result->heapObject());
+                lookup->qmlContextPropertyGetter = QQmlContextWrapper::lookupType;
+            }
+            return result->asReturnedValue();
         }
 
         // Fall through
@@ -291,11 +294,7 @@ ReturnedValue QQmlContextWrapper::getPropertyAndBase(const QQmlContextWrapper *r
                     QQmlData *ddata = QQmlData::get(scopeObject, false);
                     if (ddata && ddata->propertyCache) {
                         ScopedValue val(scope, base ? *base : Value::fromReturnedValue(QV4::QObjectWrapper::wrap(v4, scopeObject)));
-                        const QObjectWrapper *That = static_cast<const QObjectWrapper *>(val->objectValue());
-                        lookup->qobjectLookup.ic = That->internalClass();
-                        lookup->qobjectLookup.propertyCache = ddata->propertyCache;
-                        lookup->qobjectLookup.propertyCache->addref();
-                        lookup->qobjectLookup.propertyData = propertyData;
+                        QV4::setupQObjectLookup(lookup, ddata, propertyData, val->objectValue());
                         lookup->qmlContextPropertyGetter = QQmlContextWrapper::lookupScopeObjectProperty;
                     }
                 }
@@ -323,11 +322,8 @@ ReturnedValue QQmlContextWrapper::getPropertyAndBase(const QQmlContextWrapper *r
                         QQmlData *ddata = QQmlData::get(context->contextObject, false);
                         if (ddata && ddata->propertyCache) {
                             ScopedValue val(scope, base ? *base : Value::fromReturnedValue(QV4::QObjectWrapper::wrap(v4, context->contextObject)));
-                            const QObjectWrapper *That = static_cast<const QObjectWrapper *>(val->objectValue());
-                            lookup->qobjectLookup.ic = That->internalClass();
-                            lookup->qobjectLookup.propertyCache = ddata->propertyCache;
-                            lookup->qobjectLookup.propertyCache->addref();
-                            lookup->qobjectLookup.propertyData = propertyData;
+                            QV4::setupQObjectLookup(lookup, ddata, propertyData,
+                                                    val->objectValue());
                             lookup->qmlContextPropertyGetter = contextGetterFunction;
                         }
                     } else if (originalLookup) {
@@ -463,9 +459,9 @@ ReturnedValue QQmlContextWrapper::resolveQmlContextPropertyLookupGetter(Lookup *
                 return static_cast<Heap::CallContext *>(ctx)->locals[index].asReturnedValue();
         }
 
-        // Skip only block contexts within the current call context.
+        // Skip only block and call contexts.
         // Other contexts need a regular QML property lookup. See below.
-        if (ctx->type != Heap::ExecutionContext::Type_BlockContext)
+        if (ctx->type != Heap::ExecutionContext::Type_BlockContext && ctx->type != Heap::ExecutionContext::Type_CallContext)
             break;
     }
 
@@ -539,7 +535,6 @@ ReturnedValue QQmlContextWrapper::lookupIdObject(Lookup *l, ExecutionEngine *eng
 
 ReturnedValue QQmlContextWrapper::lookupScopeObjectProperty(Lookup *l, ExecutionEngine *engine, Value *base)
 {
-    Q_UNUSED(base)
     Scope scope(engine);
     Scoped<QmlContext> qmlContext(scope, engine->qmlContext());
     if (!qmlContext)
@@ -560,12 +555,15 @@ ReturnedValue QQmlContextWrapper::lookupScopeObjectProperty(Lookup *l, Execution
     };
 
     ScopedValue obj(scope, QV4::QObjectWrapper::wrap(engine, scopeObject));
+
+    if (base)
+        *base = obj;
+
     return QObjectWrapper::lookupGetterImpl(l, engine, obj, /*useOriginalProperty*/ true, revertLookup);
 }
 
 ReturnedValue QQmlContextWrapper::lookupContextObjectProperty(Lookup *l, ExecutionEngine *engine, Value *base)
 {
-    Q_UNUSED(base)
     Scope scope(engine);
     Scoped<QmlContext> qmlContext(scope, engine->qmlContext());
     if (!qmlContext)
@@ -590,6 +588,10 @@ ReturnedValue QQmlContextWrapper::lookupContextObjectProperty(Lookup *l, Executi
     };
 
     ScopedValue obj(scope, QV4::QObjectWrapper::wrap(engine, contextObject));
+
+    if (base)
+        *base = obj;
+
     return QObjectWrapper::lookupGetterImpl(l, engine, obj, /*useOriginalProperty*/ true, revertLookup);
 }
 
@@ -654,6 +656,27 @@ ReturnedValue QQmlContextWrapper::lookupInParentContextHierarchy(Lookup *l, Exec
     return Encode::undefined();
 }
 
+ReturnedValue QQmlContextWrapper::lookupType(Lookup *l, ExecutionEngine *engine, Value *base)
+{
+    Scope scope(engine);
+    Scoped<QmlContext> qmlContext(scope, engine->qmlContext());
+    if (!qmlContext)
+        return QV4::Encode::undefined();
+
+    QObject *scopeObject = qmlContext->qmlScope();
+    if (scopeObject && QQmlData::wasDeleted(scopeObject))
+        return QV4::Encode::undefined();
+
+    Heap::Object *heapObject = l->qmlTypeLookup.qmlTypeWrapper;
+    if (static_cast<Heap::QQmlTypeWrapper *>(heapObject)->object != scopeObject) {
+        l->qmlTypeLookup.qmlTypeWrapper = nullptr;
+        l->qmlContextPropertyGetter = QQmlContextWrapper::resolveQmlContextPropertyLookupGetter;
+        return QQmlContextWrapper::resolveQmlContextPropertyLookupGetter(l, engine, base);
+    }
+
+    return Value::fromHeapObject(heapObject).asReturnedValue();
+}
+
 void Heap::QmlContext::init(QV4::ExecutionContext *outerContext, QV4::QQmlContextWrapper *qml)
 {
     Heap::ExecutionContext::init(Heap::ExecutionContext::Type_QmlContext);
