@@ -65,6 +65,8 @@ const char *STEPACTION_QMLFILE = "stepAction.qml";
 const char *BREAKPOINTRELOCATION_QMLFILE = "breakpointRelocation.qml";
 const char *ENCODEQMLSCOPE_QMLFILE = "encodeQmlScope.qml";
 const char *BREAKONANCHOR_QMLFILE = "breakOnAnchor.qml";
+const char *BREAKPOINTIDS_QMLFILE = "breakPointIds.qml";
+const char *LETCONSTLOCALS_QMLFILE = "letConstLocals.qml";
 
 #undef QVERIFY
 #define QVERIFY(statement) \
@@ -156,6 +158,9 @@ private slots:
     void encodeQmlScope();
     void breakOnAnchor();
 
+    void breakPointIds();
+    void letConstLocals();
+
 private:
     ConnectResult init(bool qmlscene, const QString &qmlFile = QString(TEST_QMLFILE),
                     bool blockMode = true, bool restrictServices = false);
@@ -165,10 +170,11 @@ private:
     void targetData();
     bool waitForClientSignal(const char *signal, int timeout = 30000);
     void checkVersionParameters();
+    int setBreakPoint(const QString &file, int sourceLine, bool enabled);
+    void clearBreakPoint(int id);
 };
 
 
-
 void tst_QQmlDebugJS::initTestCase()
 {
     QQmlDebugTest::initTestCase();
@@ -566,7 +572,8 @@ void tst_QQmlDebugJS::changeBreakpoint()
 
     int sourceLine2 = 37;
     int sourceLine1 = 38;
-    QCOMPARE(init(qmlscene, CHANGEBREAKPOINT_QMLFILE), ConnectSuccess);
+    const QString file = QLatin1String(CHANGEBREAKPOINT_QMLFILE);
+    QCOMPARE(init(qmlscene, file), ConnectSuccess);
 
     bool isStopped = false;
     QObject::connect(m_client.data(), &QV4DebugClient::stopped, this, [&]() { isStopped = true; });
@@ -589,27 +596,13 @@ void tst_QQmlDebugJS::changeBreakpoint()
         return breakpointsHit[0].toInt();
     };
 
-    auto setBreakPoint = [&](int sourceLine, bool enabled) {
-        int id = -1;
-        auto connection = QObject::connect(m_client.data(), &QV4DebugClient::result, [&]() {
-            id = extractBody().value("breakpoint").toInt();
-        });
-
-        m_client->setBreakpoint(QLatin1String(CHANGEBREAKPOINT_QMLFILE), sourceLine, -1, enabled);
-        bool success = QTest::qWaitFor([&]() { return id >= 0; });
-        Q_UNUSED(success);
-
-        QObject::disconnect(connection);
-        return id;
-    };
-
     //The breakpoints are in a timer loop so we can set them after connect().
     //Furthermore the breakpoints should be hit in the right order because setting of breakpoints
     //can only occur in the QML event loop. (see QCOMPARE for sourceLine2 below)
-    const int breakpoint1 = setBreakPoint(sourceLine1, false);
+    const int breakpoint1 = setBreakPoint(file, sourceLine1, false);
     QVERIFY(breakpoint1 >= 0);
 
-    const int breakpoint2 = setBreakPoint(sourceLine2, true);
+    const int breakpoint2 = setBreakPoint(file, sourceLine2, true);
     QVERIFY(breakpoint2 >= 0);
 
     auto verifyBreakpoint = [&](int sourceLine, int breakpointId) {
@@ -1026,6 +1019,96 @@ void tst_QQmlDebugJS::breakOnAnchor()
     QCOMPARE(breaks, 2);
 }
 
+void tst_QQmlDebugJS::breakPointIds()
+{
+    QString file(BREAKPOINTIDS_QMLFILE);
+    QCOMPARE(init(true, file), ConnectSuccess);
+
+    int breaks = 0;
+    int breakPointIds[] = { -1, -1, -1, -1, -1, -1};
+
+    QObject::connect(m_client.data(), &QV4DebugClient::stopped, this, [&]() {
+        const QJsonObject body = m_client->response().body.toObject();
+        QCOMPARE(body.value("sourceLine").toInt(), breaks + 4);
+        const QJsonArray breakpointsHit = body.value("breakpoints").toArray();
+        QVERIFY(breakpointsHit.size() > 0);
+        QCOMPARE(breakpointsHit[0].toInt(), breakPointIds[breaks]);
+        ++breaks;
+        m_client->continueDebugging(QV4DebugClient::Continue);
+    });
+
+    for (int i = 0; i < 6; ++i)
+        breakPointIds[i] = setBreakPoint(file, i + 4, true);
+
+    clearBreakPoint(breakPointIds[2]);
+    breakPointIds[2] = setBreakPoint(file, 6, true);
+
+    QTRY_COMPARE(m_process->state(), QProcess::Running);
+    m_client->connect();
+
+    QTRY_COMPARE(m_process->state(), QProcess::NotRunning);
+    QCOMPARE(m_process->exitStatus(), QProcess::NormalExit);
+
+    QCOMPARE(breaks, 6);
+}
+
+void tst_QQmlDebugJS::letConstLocals()
+{
+    QString file(LETCONSTLOCALS_QMLFILE);
+    QCOMPARE(init(true, file), ConnectSuccess);
+
+    QObject::connect(m_client.data(), &QV4DebugClient::stopped, this, [&]() {
+        m_client->frame();
+    });
+
+    int numScopes = 0;
+    QString expectedMembers = QStringLiteral("abcde");
+    QObject::connect(m_client.data(), &QV4DebugClient::result, this, [&]() {
+        const auto value = m_client->response();
+        if (value.command == QStringLiteral("frame")) {
+            const auto scopes = value.body.toObject().value(QStringLiteral("scopes")).toArray();
+            for (const auto &scope : scopes) {
+                const auto scopeObject = scope.toObject();
+                const int type = scopeObject.value("type").toInt();
+                if (type == 1 || type == 4) {
+                    m_client->scope(scopeObject.value("index").toInt());
+                    ++numScopes;
+                }
+            }
+            QVERIFY(numScopes > 0);
+        } else if (value.command == QStringLiteral("scope")) {
+            const auto props = value.body.toObject().value(QStringLiteral("object")).toObject()
+                    .value(QStringLiteral("properties")).toArray();
+            for (const auto &prop : props) {
+                const auto propObj = prop.toObject();
+                const QString name = propObj.value(QStringLiteral("name")).toString();
+                if (name == QStringLiteral("onCompleted"))
+                    continue;
+                QVERIFY(name.length() == 1);
+                auto i = expectedMembers.indexOf(name.at(0));
+                QVERIFY(i != -1);
+                expectedMembers.remove(i, 1);
+                QCOMPARE(propObj.value(QStringLiteral("type")).toString(),
+                         QStringLiteral("number"));
+                QCOMPARE(propObj.value(QStringLiteral("value")).toInt(),
+                         int(name.at(0).toLatin1()));
+            }
+            if (--numScopes == 0) {
+                QVERIFY(expectedMembers.isEmpty());
+                m_client->continueDebugging(QV4DebugClient::Continue);
+            }
+        }
+    });
+
+    setBreakPoint(file, 10, true);
+
+    QTRY_COMPARE(m_process->state(), QProcess::Running);
+    m_client->connect();
+
+    QTRY_COMPARE(m_process->state(), QProcess::NotRunning);
+    QCOMPARE(m_process->exitStatus(), QProcess::NormalExit);
+}
+
 QList<QQmlDebugClient *> tst_QQmlDebugJS::createClients()
 {
     m_client = new QV4DebugClient(m_connection);
@@ -1054,6 +1137,35 @@ void tst_QQmlDebugJS::checkVersionParameters()
     QCOMPARE(body.value("ChangeBreakpoint").toBool(), true);
 }
 
+int tst_QQmlDebugJS::setBreakPoint(const QString &file, int sourceLine, bool enabled)
+{
+    int id = -1;
+    auto connection = QObject::connect(m_client.data(), &QV4DebugClient::result, [&]() {
+        id = m_client->response().body.toObject().value("breakpoint").toInt();
+    });
+
+    m_client->setBreakpoint(file, sourceLine, -1, enabled);
+    bool success = QTest::qWaitFor([&]() { return id >= 0; });
+    Q_UNUSED(success);
+
+    QObject::disconnect(connection);
+    return id;
+}
+
+void tst_QQmlDebugJS::clearBreakPoint(int id)
+{
+    bool ok = false;
+    auto connection = QObject::connect(m_client.data(), &QV4DebugClient::result, [&]() {
+        ok = true;
+    });
+
+    m_client->clearBreakpoint(id);
+    bool success = QTest::qWaitFor([&]() { return ok; });
+    Q_UNUSED(success);
+
+    QObject::disconnect(connection);
+}
+
 QTEST_MAIN(tst_QQmlDebugJS)
 
 #include "tst_qqmldebugjs.moc"
