@@ -3,8 +3,8 @@
 
 /*---
 esid: sec-array.isarray
-es5id: 15.4.3.2-1-1
-description: Array.isArray applied to boolean primitive
+es5id: 15.4.3.2-1-9
+description: Array.isArray applied to Date object
 ---*/
 
-assert.sameValue(Array.isArray(true), false, 'Array.isArray(true)');
+assert.sameValue(Array.isArray(new Date()), false, 'Array.isArray(new Date())');
