@@ -2,11 +2,11 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.3.5.4_2-67gs
+es5id: 15.3.5.4_2-71gs
 description: >
     Strict mode - checking access to strict function caller from
     non-strict function (strict function declaration called by
-    Function.prototype.call(undefined))
+    Function.prototype.bind(null)())
 flags: [noStrict]
 ---*/
 
@@ -16,7 +16,7 @@ function f() {
 };
 
 assert.throws(TypeError, function() {
-  f.call(undefined);
+  f.bind(null)();
 });
 
 function gNonStrict() {
