@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.9-1-4
+es5id: 15.2.3.9-1
 description: >
     Object.freeze does not throw TypeError if type of first param is
-    string primitive
+    not Object
 ---*/
 
-Object.freeze("abc");
+Object.freeze(0);
