@@ -2,10 +2,12 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.11-4-10
-description: Object.isSealed returns false for all built-in objects (Boolean)
+es5id: 15.2.3.11-4-20
+description: >
+    Object.isSealed returns false for all built-in objects
+    (Error.prototype)
 ---*/
 
-var b = Object.isSealed(Boolean);
+var b = Object.isSealed(Error.prototype);
 
 assert.sameValue(b, false, 'b');
