@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.10-3-16
+es5id: 15.2.3.10-3-6
 description: >
-    Object.preventExtensions - named properties cannot be added into a
-    Boolean object
+    Object.preventExtensions - indexed properties cannot be added into
+    a Boolean object
 includes: [propertyHelper.js]
 ---*/
 
@@ -15,6 +15,6 @@ assert(Object.isExtensible(obj));
 Object.preventExtensions(obj);
 assert(!Object.isExtensible(obj));
 
-verifyNotWritable(obj, "exName", "nocheck");
+verifyNotWritable(obj, "0", "nocheck");
 
-assert(!obj.hasOwnProperty("exName"));
+assert(!obj.hasOwnProperty("0"));
