@@ -3,15 +3,14 @@
 
 /*---
 info: |
-    If B = 11110xxx (n = 4) and (string.charAt(k + 4) and
+    If B = 1110xxxx (n = 3) and (string.charAt(k + 4) and
     string.charAt(k + 5)) or (string.charAt(k + 7) and
-    string.charAt(k + 8)) or (string.charAt(k + 10) and
-    string.charAt(k + 11)) do not represent hexadecimal digits, throw URIError
-es5id: 15.1.3.1_A1.12_T2
+    string.charAt(k + 8)) do not represent hexadecimal digits, throw URIError
+es5id: 15.1.3.1_A1.11_T2
 es6id: 18.2.6.2
 esid: sec-decodeuri-encodeduri
 description: >
-    Complex tests, string.charAt(k + 7) and string.charAt(k + 7)  do
+    Complex tests, string.charAt(k + 7) and string.charAt(k + 8)  do
     not represent hexadecimal digits
 ---*/
 
@@ -26,7 +25,7 @@ var interval = [
 for (var indexI = 0; indexI < interval.length; indexI++) {
   for (var indexJ = interval[indexI][0]; indexJ <= interval[indexI][1]; indexJ++) {
     try {
-      decodeURI("%F0" + "%A0%" + String.fromCharCode(indexJ, indexJ) + "%A0");
+      decodeURI("%E0" + "%A0%" + String.fromCharCode(indexJ, indexJ));
       result = false;
     } catch (e) {
       if ((e instanceof URIError) !== true) {
@@ -37,5 +36,5 @@ for (var indexI = 0; indexI < interval.length; indexI++) {
 }
 
 if (result !== true) {
-  $ERROR('#1: If B = 11110xxx (n = 4) and (string.charAt(k + 7) and string.charAt(k + 8)) do not represent hexadecimal digits, throw URIError');
+  $ERROR('#1: If B = 1110xxxx (n = 3) and (string.charAt(k + 7) and  string.charAt(k + 8)) do not represent hexadecimal digits, throw URIError');
 }
