@@ -2,15 +2,15 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: "StringLiteral :: \"\\\" or '\\' is not correct"
-es5id: 7.8.4_A3.1_T2
-description: Checking if execution of "'\'" fails
+info: NonEscapeSequence is not EscapeCharacter
+es5id: 7.8.4_A4.3_T2
+description: "EscapeCharacter :: DecimalDigits :: 7"
 negative:
   phase: parse
   type: SyntaxError
+flags: [onlyStrict]
 ---*/
 
 throw "Test262: This statement should not be evaluated.";
 
-//CHECK#1
-'\'
+"\7"
