@@ -2,10 +2,11 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 13.1-13gs
+es5id: 13.1-4gs
 description: >
-    StrictMode - SyntaxError is thrown if 'arguments' occurs as the
-    Identifier of a FunctionDeclaration
+    Strict Mode - SyntaxError is thrown if the identifier 'arguments'
+    appears within a FormalParameterList of a strict mode
+    FunctionExpression
 negative:
   phase: parse
   type: SyntaxError
@@ -14,4 +15,4 @@ flags: [onlyStrict]
 
 throw "Test262: This statement should not be evaluated.";
 
-function arguments() { };
+var _13_1_4_fun = function (arguments) { };
