@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 12.2.1-1gs
+es5id: 12.2.1-4gs
 description: >
-    Strict Mode - SyntaxError is thrown if a VariableDeclaration
-    occurs within strict code and its Identifier is eval
+    Strict Mode - SyntaxError is thrown if a VariableDeclarationNoIn
+    occurs within strict code and its Identifier is arguments
 negative:
   phase: parse
   type: SyntaxError
@@ -14,4 +14,4 @@ flags: [onlyStrict]
 
 throw "Test262: This statement should not be evaluated.";
 
-for (var eval in arrObj) { }
+var arguments;
