@@ -5,10 +5,9 @@
 info: |
     The with statement adds a computed object to the front of the
     scope chain of the current execution context
-es5id: 12.10_A1.2_T1
+es5id: 12.10_A1.6_T1
 description: >
-    Calling a function without "with" statement when the statement
-    itself is declared within the function declaration, leading to
+    Using "with" statement within another "with" statement, leading to
     normal completion
 flags: [noStrict]
 ---*/
@@ -42,7 +41,7 @@ var st_parseFloat = "parseFloat";
 var st_isNaN = "isNaN";
 var st_isFinite = "isFinite";
 
-var f = function(){
+with(myObj){
   with(myObj){
     st_p1 = p1;
     st_p2 = p2;
@@ -62,7 +61,6 @@ var f = function(){
     var value = 'value';
   }
 }
-f();
 
 if(!(p1 === 1)){
   $ERROR('#1: p1 === 1. Actual:  p1 ==='+ p1  );
@@ -76,10 +74,8 @@ if(!(p3 === 3)){
   $ERROR('#3: p3 === 3. Actual:  p3 ==='+ p3  );
 }
 
-try {
-  p4;
-  $ERROR('#4: p4 is not defined');
-} catch(e) {    
+if(!(p4 === "x4")){
+  $ERROR('#4: p4 === "x4". Actual:  p4 ==='+ p4  );
 }
 
 if(!(p5 === "x5")){
@@ -134,11 +130,8 @@ if(!(st_isFinite !== isFinite)){
   $ERROR('#17: myObj.isFinite !== isFinite');
 }
 
-try{
-  value;
-  $ERROR('#18: value is not defined');
-}
-catch(e){
+if(!(value === undefined)){
+  $ERROR('#18: value === undefined. Actual:  value ==='+ value  );
 }
 
 if(!(myObj.value === "value")){
