@@ -36,52 +36,44 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '15.9.5.14.js';
+gTestfile = '15.9.1.13-1.js';
 
 /**
-   File Name:          15.9.5.14.js
-   ECMA Section:       15.9.5.14
-   Description:        Date.prototype.getHours
-   1.  Let t be this time value.
-   2.  If t is NaN, return NaN.
-   3.  Return HourFromTime(LocalTime(t)).
+   File Name:          15.9.1.13-1.js
+   ECMA Section:       15.9.1.1 MakeDate(day, time)
+   Description:
 
-   Author:             christine@netscape.com
-   Date:               12 november 1997
+   The operator MakeDate calculates a number of milliseconds from its
+   two arguments, which must be ECMAScript number values. This
+   operator functions as follows:
+
+   1. If day is not finite or time is not finite, return NaN.
+
+   2. Compute day * msPerDay + time.
+
+   3. Return Result(2).
 */
-
-var SECTION = "15.9.5.14";
-var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Date.prototype.getHours()";
-
-writeHeaderToLog( SECTION + " "+ TITLE);
-
-addTestCase( TIME_NOW );
-addTestCase( TIME_0000 );
-addTestCase( TIME_1970 );
-addTestCase( TIME_1900 );
-addTestCase( TIME_2000 );
-addTestCase( UTC_FEB_29_2000 );
-addTestCase( UTC_JAN_1_2005 );
 
 new TestCase( SECTION,
-	      "(new Date(NaN)).getHours()",
-	      NaN,
-	      (new Date(NaN)).getHours() );
+              "MakeDate(Number.POSITIVE_INFINITY, 0)",
+              Number.NaN,
+              MakeDate(Number.POSITIVE_INFINITY, 0));
 
 new TestCase( SECTION,
-	      "Date.prototype.getHours.length",
-	      0,
-	      Date.prototype.getHours.length );
+              "MakeDate(Number.NEGATIVE_INFINITY, 0)",
+              Number.NaN,
+              MakeDate(Number.NEGATIVE_INFINITY, 0));
+
+new TestCase( SECTION,
+              "MakeDate(0, Number.POSITIVE_INFINITY)",
+              Number.NaN,
+              MakeDate(0, Number.POSITIVE_INFINITY));
+
+new TestCase( SECTION,
+              "MakeDate(0, Number.NEGATIVE_INFINITY)",
+              Number.NaN,
+              MakeDate(0, Number.NEGATIVE_INFINITY));
+
 test();
 
-function addTestCase( t ) {
-  for ( h = 0; h < 24; h+=4 ) {
-    t += msPerHour;
-    new TestCase( SECTION,
-		  "(new Date("+t+")).getHours()",
-		  HourFromTime((LocalTime(t))),
-		  (new Date(t)).getHours() );
-  }
-}
