@@ -36,29 +36,42 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '7.7.1.js';
+gTestfile = '7.7.3-2.js';
 
 /**
-   File Name:          7.7.1.js
-   ECMA Section:       7.7.1 Null Literals
+   File Name:          7.7.3-2.js
+   ECMA Section:       7.7.3 Numeric Literals
 
-   Description:        NullLiteral::
-   null
+   Description:
 
+   This is a regression test for
+   http://scopus.mcom.com/bugsplat/show_bug.cgi?id=122884
 
-   The value of the null literal null is the sole value
-   of the Null type, namely null.
+   Waldemar's comments:
+
+   A numeric literal that starts with either '08' or '09' is interpreted as a
+   decimal literal; it should be an error instead.  (Strictly speaking, according
+   to ECMA v1 such literals should be interpreted as two integers -- a zero
+   followed by a decimal number whose first digit is 8 or 9, but this is a bug in
+   ECMA that will be fixed in v2.  In any case, there is no place in the grammar
+   where two consecutive numbers would be legal.)
 
    Author:             christine@netscape.com
-   Date:               21 october 1997
+   Date:               15 june 1998
+
 */
-var SECTION = "7.7.1";
+var SECTION = "7.7.3-2";
 var VERSION = "ECMA_1";
+var TITLE   = "Numeric Literals";
+var BUGNUMBER="122884";
+
 startTest();
-var TITLE   = "Null Literals";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-new TestCase( SECTION, "null",     null,        null);
+new TestCase( SECTION,
+	      "9",
+	      9,
+	      9 );
 
 test();
