@@ -36,39 +36,28 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '12.2-1.js';
+gTestfile = '12.6.3-2.js';
 
 /**
-   File Name:          12.2-1.js
-   ECMA Section:       The variable statement
-   Description:
+   File Name:          12.6.3-2.js
+   ECMA Section:       12.6.3 The for...in Statement
+   Description:        Check the Boolean Object
 
-   If the variable statement occurs inside a FunctionDeclaration, the
-   variables are defined with function-local scope in that function, as
-   described in section 10.1.3. Otherwise, they are defined with global
-   scope, that is, they are created as members of the global object, as
-   described in section 0. Variables are created when the execution scope
-   is entered. A Block does not define a new execution scope. Only Program and
-   FunctionDeclaration produce a new scope. Variables are initialized to the
-   undefined value when created. A variable with an Initializer is assigned
-   the value of its AssignmentExpression when the VariableStatement is executed,
-   not when the variable is created.
 
    Author:             christine@netscape.com
-   Date:               12 november 1997
+   Date:               11 september 1997
 */
 
-var SECTION = "12.2-1";
+var SECTION = "12.6.3-2";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "The variable statement";
+var TITLE   = "The for..in statement";
 
-writeHeaderToLog( SECTION +" "+ TITLE);
+writeHeaderToLog( SECTION + " "+ TITLE);
 
-new TestCase(    "SECTION",
-		 "var x = 3; function f() { var a = x; var x = 23; return a; }; f()",
-		 void 0,
-		 eval("var x = 3; function f() { var a = x; var x = 23; return a; }; f()") );
+new TestCase(   SECTION,
+		"Boolean.prototype.foo = 34; for ( j in Boolean ) Boolean[j]",
+		34,
+		eval("Boolean.prototype.foo = 34; for ( j in Boolean ) Boolean[j] ") );
 
 test();
-
